@@ -38,6 +38,7 @@ namespace_subnet_driver_opts = [
 ]
 
 oslo_cfg.CONF.register_opts(namespace_subnet_driver_opts, "namespace_subnet")
+TAGS = oslo_cfg.CONF.neutron_defaults.resource_tags
 
 
 class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
@@ -102,85 +103,77 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
         try:
             os_net.delete_network(net_id)
         except os_exc.ConflictException:
-            LOG.exception("One or more ports in use on the network %s. "
-                          "Deleting leftovers ports before retrying", net_id)
-            leftover_ports = os_net.ports(network_id=net_id)
-            for leftover_port in leftover_ports:
-                # NOTE(dulek): '' is there because Neutron seems to unset
-                #              device_owner on detach.
-                if leftover_port.device_owner not in ['', 'trunk:subport',
-                                                      kl_const.DEVICE_OWNER]:
-                    continue
-                try:
-                    # NOTE(gryf): there is unlikely, that we get an exception
-                    # like PortNotFound or something, since openstacksdk
-                    # doesn't raise an exception if port doesn't exists nor
-                    # return any information.
-                    os_net.delete_port(leftover_port.id)
-                except os_exc.SDKException as e:
-                    if "currently a subport for trunk" in str(e):
-                        if leftover_port.status == "DOWN":
-                            LOG.warning("Port %s is in DOWN status but still "
-                                        "associated to a trunk. This should "
-                                        "not happen. Trying to delete it from "
-                                        "the trunk.", leftover_port.id)
-                        # Get the trunk_id from the error message
-                        trunk_id = (
-                            str(e).split('trunk')[1].split('.')[0].strip())
-                        try:
-                            os_net.delete_trunk_subports(
-                                trunk_id, [{'port_id': leftover_port.id}])
-                        except os_exc.NotFoundException:
-                            LOG.debug("Port %s already removed from trunk %s",
-                                      leftover_port['id'], trunk_id)
-                    else:
-                        LOG.exception("Unexpected error deleting leftover "
-                                      "port %s. Skiping it and continue with "
-                                      "the other rest.", leftover_port.id)
-
+            LOG.warning("One or more ports in use on the network %s. "
+                        "Deleting leftovers ports before retrying", net_id)
+            # NOTE(dulek): '' is there because Neutron seems to unset
+            #              device_owner on detach.
+            leftover_ports = [p for p in os_net.ports(network_id=net_id)
+                              if p.device_owner in
+                              ['', 'trunk:subport', kl_const.DEVICE_OWNER]]
+            c_utils.delete_ports(leftover_ports)
             raise exceptions.ResourceNotReady(net_id)
         except os_exc.SDKException:
             LOG.exception("Error deleting network %s.", net_id)
             raise
 
-    def create_network(self, ns_name, project_id):
+    def create_network(self, ns, project_id):
         os_net = clients.get_network_client()
-        net_name = 'ns/' + ns_name + '-net'
-        tags = oslo_cfg.CONF.neutron_defaults.resource_tags
-        if tags:
-            networks = os_net.networks(name=net_name, tags=tags)
-        else:
-            networks = os_net.networks(name=net_name)
+        ns_name = ns['metadata']['name']
+        ns_uid = ns['metadata']['uid']
+        net_name = c_utils.get_resource_name(ns_name, ns_uid)
+        old_net_name = c_utils.get_resource_name(ns_name, prefix='ns/',
+                                                 suffix='-net')
+        # TODO(gryf): remove old_net_name support in next release, and precise
+        # the query by adding additional query parameter 'description' which
+        # should contain namespace uid.
+        networks = os_net.networks(name=(net_name, old_net_name))
+        tags = ",".join(TAGS)
 
         try:
             # NOTE(ltomasbo): only one network must exists
-            return next(networks).id
-        except StopIteration:
+            net = next(networks)
+            # NOTE(gryf): It might happen, that network has been created, but
+            # for some reason tagging has failed.
+            if TAGS and not set(TAGS).issubset(set(net.tags)):
+                c_utils.tag_neutron_resources([net], exceptions=True)
+            return net.id
+        except (StopIteration, ValueError):
             LOG.debug('Network does not exist. Creating.')
 
-        # create network with namespace as name
+        mtu_cfg = oslo_cfg.CONF.neutron_defaults.network_device_mtu
+        attrs = {'name': net_name, 'project_id': project_id,
+                 'description': tags}
+        if mtu_cfg:
+            attrs['mtu'] = mtu_cfg
+
         try:
-            neutron_net = os_net.create_network(name=net_name,
-                                                project_id=project_id)
-            c_utils.tag_neutron_resources([neutron_net])
+            net = os_net.create_network(**attrs)
         except os_exc.SDKException:
             LOG.exception("Error creating neutron resources for the namespace "
                           "%s", ns_name)
             raise
-        return neutron_net.id
+        c_utils.tag_neutron_resources([net], exceptions=True)
+        return net.id
 
-    def create_subnet(self, ns_name, project_id, net_id):
+    def create_subnet(self, ns, project_id, net_id):
         os_net = clients.get_network_client()
-        subnet_name = "ns/" + ns_name + "-subnet"
-        tags = oslo_cfg.CONF.neutron_defaults.resource_tags
-        if tags:
-            subnets = os_net.subnets(name=subnet_name, tags=tags)
-        else:
-            subnets = os_net.subnets(name=subnet_name)
+        ns_name = ns['metadata']['name']
+        ns_uid = ns['metadata']['uid']
+        tags = ",".join(TAGS)
+
+        # NOTE(gryf): assumption is, that all the subnets (well, currently
+        # only one) in specific k8s namespaces are under exactly one network,
+        # which have proper namespace uid in its description, so there is no
+        # need to put it on the subnet as well.
+        subnet_name = c_utils.get_resource_name(ns_name, ns_uid)
+        subnets = os_net.subnets(network_id=net_id)
 
         try:
             # NOTE(ltomasbo): only one subnet must exists
             subnet = next(subnets)
+            # NOTE(gryf): same situation as in networks.
+            if TAGS and not set(TAGS).issubset(set(subnet.tags)):
+                c_utils.tag_neutron_resources([subnet], exceptions=True)
             return subnet.id, subnet.cidr
         except StopIteration:
             LOG.debug('Subnet does not exist. Creating.')
@@ -191,6 +184,7 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
         try:
             neutron_subnet = (os_net
                               .create_subnet(network_id=net_id,
+                                             description=tags,
                                              ip_version=ip_version,
                                              name=subnet_name,
                                              enable_dhcp=False,
@@ -201,7 +195,7 @@ class NamespacePodSubnetDriver(default_subnet.DefaultPodSubnetDriver):
                       "raising ResourceNotReady to retry subnet creation "
                       "for %s", subnet_name)
             raise exceptions.ResourceNotReady(subnet_name)
-        c_utils.tag_neutron_resources([neutron_subnet])
+        c_utils.tag_neutron_resources([neutron_subnet], exceptions=True)
 
         return neutron_subnet.id, neutron_subnet.cidr
 
