@@ -32,60 +32,45 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
     def __init__(self):
         super(NamespaceHandler, self).__init__()
         self._drv_project = drivers.NamespaceProjectDriver.get_instance()
-        self._upgrade_crds()
 
-    def _upgrade_crds(self):
-        k8s = clients.get_kubernetes_client()
-        try:
-            net_crds = k8s.get(constants.K8S_API_CRD_KURYRNETS)
-            namespaces = k8s.get(constants.K8S_API_NAMESPACES)
-        except exceptions.K8sResourceNotFound:
-            return
-        except exceptions.K8sClientException:
-            LOG.warning("Error retriving namespace information")
-            raise
-
-        ns_dict = {'ns-' + ns['metadata']['name']: ns
-                   for ns in namespaces.get('items')}
-
-        for net_crd in net_crds.get('items'):
-            try:
-                ns = ns_dict[net_crd['metadata']['name']]
-            except KeyError:
-                # Note(ltomasbo): The CRD does not have an associated
-                # namespace. It must be deleted
-                LOG.debug('No namespace associated, deleting kuryrnet crd: '
-                          '%s', net_crd)
-            else:
-                try:
-                    ns_net_annotations = ns['metadata']['annotations'][
-                        constants.K8S_ANNOTATION_NET_CRD]
-                except KeyError:
-                    LOG.debug('Namespace associated is not annotated: %s', ns)
-                else:
-                    LOG.debug('Removing annotation: %', ns_net_annotations)
-                    k8s.remove_annotations(ns['metadata']['selfLink'],
-                                           constants.K8S_ANNOTATION_NET_CRD)
-            try:
-                k8s.delete(net_crd['metadata']['selfLink'])
-            except exceptions.K8sResourceNotFound:
-                LOG.debug('Kuryrnet object already deleted: %s', net_crd)
-
-    def on_present(self, namespace):
+    def on_present(self, namespace, *args, **kwargs):
         ns_labels = namespace['metadata'].get('labels', {})
         ns_name = namespace['metadata']['name']
+
         kns_crd = self._get_kns_crd(ns_name)
         if kns_crd:
             LOG.debug("Previous CRD existing at the new namespace.")
             self._update_labels(kns_crd, ns_labels)
             return
 
+        if not self._handle_namespace(ns_name):
+            LOG.debug("Namespace %s has no Pods that should be handled. "
+                      "Skipping event.", ns_name)
+            return
+
         try:
-            self._add_kuryrnetwork_crd(ns_name, ns_labels)
+            self._add_kuryrnetwork_crd(namespace, ns_labels)
         except exceptions.K8sClientException:
             LOG.exception("Kuryrnetwork CRD creation failed.")
             raise exceptions.ResourceNotReady(namespace)
 
+    def _handle_namespace(self, namespace):
+        """Evaluate if the Namespace should be handled
+
+        Fetches all the Pods in the Namespace and check
+        if there is any Pod in that Namespace on Pods Network.
+
+        :param namespace: Namespace name
+        :returns: True if the Namespace resources should be
+                  created, False if otherwise.
+        """
+        kubernetes = clients.get_kubernetes_client()
+        pods = kubernetes.get(
+                '{}/namespaces/{}/pods'.format(
+                    constants.K8S_API_BASE, namespace))
+        return any(not utils.is_host_network(pod)
+                   for pod in pods.get('items', []))
+
     def _update_labels(self, kns_crd, ns_labels):
         kns_status = kns_crd.get('status')
         if kns_status:
@@ -97,7 +82,7 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
         kubernetes = clients.get_kubernetes_client()
         LOG.debug('Patching KuryrNetwork CRD %s', kns_crd)
         try:
-            kubernetes.patch_crd('spec', kns_crd['metadata']['selfLink'],
+            kubernetes.patch_crd('spec', utils.get_res_link(kns_crd),
                                  {'nsLabels': ns_labels})
         except exceptions.K8sResourceNotFound:
             LOG.debug('KuryrNetwork CRD not found %s', kns_crd)
@@ -119,6 +104,7 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
         return kuryrnetwork_crd
 
     def _add_kuryrnetwork_crd(self, namespace, ns_labels):
+        ns_name = namespace['metadata']['name']
         project_id = self._drv_project.get_project(namespace)
         kubernetes = clients.get_kubernetes_client()
 
@@ -126,25 +112,25 @@ class NamespaceHandler(k8s_base.ResourceEventHandler):
             'apiVersion': 'openstack.org/v1',
             'kind': 'KuryrNetwork',
             'metadata': {
-                'name': namespace,
+                'name': ns_name,
                 'finalizers': [constants.KURYRNETWORK_FINALIZER],
             },
             'spec': {
-                'nsName': namespace,
+                'nsName': ns_name,
                 'projectId': project_id,
                 'nsLabels': ns_labels,
             }
         }
         try:
             kubernetes.post('{}/{}/kuryrnetworks'.format(
-                constants.K8S_API_CRD_NAMESPACES, namespace), kns_crd)
+                constants.K8S_API_CRD_NAMESPACES, ns_name), kns_crd)
         except exceptions.K8sClientException:
             LOG.exception("Kubernetes Client Exception creating kuryrnetwork "
                           "CRD.")
             raise
 
     def is_ready(self, quota):
-        if not (utils.has_kuryr_crd(constants.K8S_API_CRD_KURYRNETS) and
+        if not (utils.has_kuryr_crd(constants.K8S_API_CRD_KURYRNETWORKS) and
                 self._check_quota(quota)):
             LOG.error('Marking NamespaceHandler as not ready.')
             return False
