@@ -10,12 +10,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import munch
+import eventlet
 from unittest import mock
 
 from kuryr.lib import constants as kl_const
 from kuryr.lib import exceptions as kl_exc
 from openstack import exceptions as os_exc
+from openstack.network.v2 import port as os_port
+from openstack.network.v2 import trunk as os_trunk
 from oslo_config import cfg as oslo_cfg
 
 from kuryr_kubernetes import constants
@@ -31,6 +33,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         'kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif_nested_vlan')
     def test_request_vif(self, m_to_vif):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = True
         m_driver = mock.Mock(spec=cls)
         os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
@@ -42,7 +45,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         parent_port = mock.sentinel.parent_port
         trunk_id = mock.sentinel.trunk_id
         port_id = mock.sentinel.port_id
-        port = munch.Munch({'id': port_id})
+        port = os_port.Port(id=port_id)
         port_request = {'project_id': project_id,
                         'name': mock.sentinel.name,
                         'network_id': mock.sentinel.network_id,
@@ -75,6 +78,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         'kuryr_kubernetes.os_vif_util.neutron_to_osvif_vif_nested_vlan')
     def test_request_vifs(self, m_to_vif):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = True
         m_driver = mock.Mock(spec=cls)
         os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
@@ -93,10 +97,10 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          {'segmentation_id': 2,
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
-        port = munch.Munch({'id': mock.sentinel.id})
+        port = os_port.Port(id=mock.sentinel.id)
         vif = mock.sentinel.vif
 
-        bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
+        bulk_rq = [port_request for _ in range(len(subports_info))]
 
         m_driver._get_parent_port.return_value = parent_port
         m_driver._get_trunk_id.return_value = trunk_id
@@ -104,9 +108,11 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                                                        subports_info)
         os_net.create_ports.return_value = (p for p in [port, port])
         m_to_vif.return_value = vif
+        semaphore = mock.MagicMock(spec=eventlet.semaphore.Semaphore(20))
 
         self.assertEqual([vif, vif], cls.request_vifs(
-            m_driver, pod, project_id, subnets, security_groups, num_ports))
+            m_driver, pod, project_id, subnets, security_groups, num_ports,
+            semaphore))
 
         m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
@@ -124,6 +130,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
 
     def test_request_vifs_no_vlans(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = False
         m_driver = mock.Mock(spec=cls)
         self.useFixture(k_fix.MockNetworkClient()).client
 
@@ -142,10 +149,11 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._create_subports_info.return_value = (port_request,
                                                        subports_info)
+        semaphore = mock.MagicMock(spec=eventlet.semaphore.Semaphore(20))
 
         self.assertEqual([], cls.request_vifs(m_driver, pod, project_id,
                                               subnets, security_groups,
-                                              num_ports))
+                                              num_ports, semaphore))
 
         m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
@@ -155,6 +163,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
 
     def test_request_vifs_bulk_creation_exception(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = True
         m_driver = mock.Mock(spec=cls)
         os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
@@ -174,17 +183,19 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
 
-        bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
+        bulk_rq = [port_request for _ in range(len(subports_info))]
 
         m_driver._get_parent_port.return_value = parent_port
         m_driver._get_trunk_id.return_value = trunk_id
         m_driver._create_subports_info.return_value = (port_request,
                                                        subports_info)
         os_net.create_ports.side_effect = os_exc.SDKException
+        semaphore = mock.MagicMock(spec=eventlet.semaphore.Semaphore(20))
 
         self.assertRaises(
             os_exc.SDKException, cls.request_vifs,
-            m_driver, pod, project_id, subnets, security_groups, num_ports)
+            m_driver, pod, project_id, subnets, security_groups, num_ports,
+            semaphore)
 
         m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
@@ -193,8 +204,10 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
             trunk_id, num_ports, unbound=True)
         os_net.create_ports.assert_called_once_with(bulk_rq)
 
-    def test_request_vifs_trunk_subports_conflict(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.delete_ports')
+    def test_request_vifs_trunk_subports_conflict(self, m_del_ports):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = True
         m_driver = mock.Mock(spec=cls)
         os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
@@ -213,9 +226,9 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          {'segmentation_id': 2,
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
-        port = munch.Munch({'id': mock.sentinel.id})
+        port = os_port.Port(id=mock.sentinel.id)
 
-        bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
+        bulk_rq = [port_request for _ in range(len(subports_info))]
 
         m_driver._get_parent_port.return_value = parent_port
         m_driver._get_trunk_id.return_value = trunk_id
@@ -223,9 +236,10 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                                                        subports_info)
         os_net.create_ports.return_value = (p for p in [port, port])
         os_net.add_trunk_subports.side_effect = os_exc.ConflictException
+        semaphore = mock.MagicMock(spec=eventlet.semaphore.Semaphore(20))
 
         self.assertEqual([], cls.request_vifs(m_driver, pod, project_id,
-                         subnets, security_groups, num_ports))
+                         subnets, security_groups, num_ports, semaphore))
 
         m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
@@ -235,10 +249,12 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         os_net.create_ports.assert_called_once_with(bulk_rq)
         os_net.add_trunk_subports.assert_called_once_with(trunk_id,
                                                           subports_info)
-        os_net.delete_port.assert_called_with(port['id'])
+        m_del_ports.assert_called_once_with([port, port])
 
-    def test_request_vifs_trunk_subports_exception(self):
+    @mock.patch('kuryr_kubernetes.controller.drivers.utils.delete_ports')
+    def test_request_vifs_trunk_subports_exception(self, m_del_ports):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = False
         m_driver = mock.Mock(spec=cls)
         os_net = self.useFixture(k_fix.MockNetworkClient()).client
 
@@ -257,9 +273,9 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                          {'segmentation_id': 2,
                           'port_id': '',
                           'segmentation_type': 'vlan'}]
-        port = munch.Munch({'id': mock.sentinel.id})
+        port = os_port.Port(id=mock.sentinel.id)
 
-        bulk_rq = {'ports': [port_request for _ in range(len(subports_info))]}
+        bulk_rq = [port_request for _ in range(len(subports_info))]
 
         m_driver._get_parent_port.return_value = parent_port
         m_driver._get_trunk_id.return_value = trunk_id
@@ -267,9 +283,10 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                                                        subports_info)
         os_net.create_ports.return_value = (p for p in [port, port])
         os_net.add_trunk_subports.side_effect = os_exc.SDKException
+        semaphore = mock.MagicMock(spec=eventlet.semaphore.Semaphore(20))
 
         self.assertEqual([], cls.request_vifs(m_driver, pod, project_id,
-                         subnets, security_groups, num_ports))
+                         subnets, security_groups, num_ports, semaphore))
 
         m_driver._get_parent_port.assert_called_once_with(pod)
         m_driver._get_trunk_id.assert_called_once_with(parent_port)
@@ -279,7 +296,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         os_net.create_ports.assert_called_once_with(bulk_rq)
         os_net.add_trunk_subports.assert_called_once_with(trunk_id,
                                                           subports_info)
-        os_net.delete_port.assert_called_with(port['id'])
+        m_del_ports.assert_called_once_with([port, port])
 
     def test_release_vif(self):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
@@ -326,6 +343,7 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
                                m_get_network_id, m_get_port_name,
                                unbound=False):
         cls = nested_vlan_vif.NestedVlanPodVIFDriver
+        cls._tag_on_creation = True
         m_driver = mock.Mock(spec=cls)
 
         pod = mock.sentinel.pod
@@ -626,8 +644,8 @@ class TestNestedVlanPodVIFDriver(test_base.TestCase):
         trunk_id = mock.sentinel.trunk_id
         vlan_ids.add('100')
 
-        port = munch.Munch({'segmentation_id': '100'})
-        trunk_obj = munch.Munch({'sub_ports': [port]})
+        port = os_port.Port(segmentation_id='100')
+        trunk_obj = os_trunk.Trunk(sub_ports=[port])
         os_net.get_trunk.return_value = trunk_obj
         self.assertEqual(vlan_ids,
                          cls._get_in_use_vlan_ids_set(m_driver, trunk_id))
