@@ -14,6 +14,8 @@
 
 from unittest import mock
 
+from openstack.network.v2 import network as os_network
+from openstack.network.v2 import subnet as os_subnet
 from oslo_config import cfg as oslo_cfg
 
 from kuryr_kubernetes.controller.drivers import base as drivers
@@ -26,6 +28,8 @@ from kuryr_kubernetes.tests import base as test_base
 from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
 
 
+@mock.patch('kuryr_kubernetes.controller.drivers.utils.'
+            'is_network_policy_enabled', mock.Mock(return_value=True))
 class TestKuryrNetworkHandler(test_base.TestCase):
 
     def setUp(self):
@@ -49,13 +53,14 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._handler = mock.MagicMock(
             spec=kuryrnetwork.KuryrNetworkHandler)
         self._handler._drv_project = mock.Mock(spec=drivers.PodProjectDriver)
-        # NOTE(ltomasbo): The KuryrNet handler is associated to the usage of
-        # namespace subnet driver,
+        # NOTE(ltomasbo): The KuryrNetwork handler is associated to the usage
+        # of namespace subnet driver,
         self._handler._drv_subnets = mock.Mock(
             spec=subnet_drv.NamespacePodSubnetDriver)
         self._handler._drv_sg = mock.Mock(spec=drivers.PodSecurityGroupsDriver)
         self._handler._drv_vif_pool = mock.MagicMock(
             spec=vif_pool.MultiVIFPool)
+        self._handler.k8s = self.useFixture(k_fix.MockK8sClient()).client
 
         self._get_project = self._handler._drv_project.get_project
         self._set_vif_driver = self._handler._drv_vif_pool.set_vif_driver
@@ -107,7 +112,8 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         subnet_id = mock.sentinel.subnet_id
         subnet_cidr = mock.sentinel.subnet_cidr
         router_id = mock.sentinel.router_id
-        ns = mock.sentinel.namespace
+        ns = {'metadata': {'uid': 'e28127f4-bc41-450a-97cf-56d9bc76d53e',
+                           'name': 'test'}}
 
         self._create_network.return_value = net_id
         self._create_subnet.return_value = (subnet_id, subnet_cidr)
@@ -120,10 +126,10 @@ class TestKuryrNetworkHandler(test_base.TestCase):
 
         self._handler._patch_kuryrnetwork_crd.assert_called()
         self._create_network.assert_called_once_with(
-            self._kuryrnet_crd['spec']['nsName'],
+            ns,
             self._kuryrnet_crd['spec']['projectId'])
         self._create_subnet.assert_called_once_with(
-            self._kuryrnet_crd['spec']['nsName'],
+            ns,
             self._kuryrnet_crd['spec']['projectId'],
             net_id)
         self._add_subnet_to_router.assert_called_once_with(subnet_id)
@@ -131,6 +137,7 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._update_ns_sg_rules.assert_called_once_with(ns)
         m_get_svc.assert_called_once()
         self._handler._update_services.assert_called_once()
+        self.assertEqual(self._handler.k8s.add_event.call_count, 4)
 
     @mock.patch.object(driver_utils, 'get_services')
     @mock.patch.object(driver_utils, 'get_namespace')
@@ -139,7 +146,8 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         subnet_id = mock.sentinel.subnet_id
         subnet_cidr = mock.sentinel.subnet_cidr
         router_id = mock.sentinel.router_id
-        ns = mock.sentinel.namespace
+        ns = {'metadata': {'uid': '843645f7-d255-4fd8-86fb-09140d57c392',
+                           'name': 'test'}}
 
         self._create_network.return_value = net_id
         self._create_subnet.return_value = (subnet_id, subnet_cidr)
@@ -157,10 +165,10 @@ class TestKuryrNetworkHandler(test_base.TestCase):
 
         self._handler._patch_kuryrnetwork_crd.assert_called()
         self._create_network.assert_called_once_with(
-            self._kuryrnet_crd['spec']['nsName'],
+            ns,
             self._kuryrnet_crd['spec']['projectId'])
         self._create_subnet.assert_called_once_with(
-            self._kuryrnet_crd['spec']['nsName'],
+            ns,
             self._kuryrnet_crd['spec']['projectId'],
             net_id)
         self._add_subnet_to_router.assert_called_once_with(subnet_id)
@@ -168,6 +176,7 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._update_ns_sg_rules.assert_called_once_with(ns)
         m_get_svc.assert_not_called()
         self._handler._update_services.assert_not_called()
+        self.assertEqual(self._handler.k8s.add_event.call_count, 4)
 
     @mock.patch.object(driver_utils, 'get_namespace')
     def test_on_present_existing(self, m_get_ns):
@@ -189,7 +198,7 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._create_network.assert_not_called()
         self._create_subnet.assert_not_called()
         self._add_subnet_to_router.assert_not_called()
-        m_get_ns.assert_not_called()
+        self._handler.k8s.add_event.assert_not_called()
 
     @mock.patch.object(driver_utils, 'get_services')
     def test_on_finalize(self, m_get_svc):
@@ -200,7 +209,6 @@ class TestKuryrNetworkHandler(test_base.TestCase):
 
         self._delete_ns_sg_rules.return_value = [crd_selector]
         m_get_svc.return_value = []
-        kubernetes = self.useFixture(k_fix.MockK8sClient()).client
 
         kuryrnetwork.KuryrNetworkHandler.on_finalize(self._handler, kns_crd)
 
@@ -209,15 +217,21 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._delete_ns_sg_rules.assert_called_once()
         m_get_svc.assert_called_once()
         self._handler._update_services.assert_called_once()
-        kubernetes.remove_finalizer.assert_called_once()
+        self._handler.k8s.remove_finalizer.assert_called_once()
+        self._handler.k8s.add_event.assert_not_called()
 
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
     @mock.patch.object(driver_utils, 'get_services')
-    def test_on_finalize_no_network(self, m_get_svc):
+    def test_on_finalize_no_network(self, m_get_svc, m_get_net_client):
         crd_selector = mock.sentinel.crd_selector
         self._delete_ns_sg_rules.return_value = [crd_selector]
         m_get_svc.return_value = []
-
-        kubernetes = self.useFixture(k_fix.MockK8sClient()).client
+        net_mock = mock.MagicMock()
+        net_mock.return_value = []
+        m_get_net_client.return_value = net_mock
+        self._handler.k8s.get.return_value = (
+            {'metadata': {'name': 'test-namespace',
+                          'uid': 'e237acaf-ea5f-4d96-b604-292268a938a1'}})
 
         kuryrnetwork.KuryrNetworkHandler.on_finalize(self._handler,
                                                      self._kuryrnet_crd)
@@ -227,7 +241,91 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._delete_ns_sg_rules.assert_called_once()
         m_get_svc.assert_called_once()
         self._handler._update_services.assert_called_once()
-        kubernetes.remove_finalizer.assert_called_once()
+        self._handler.k8s.remove_finalizer.assert_called_once()
+        self._handler.k8s.add_event.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    @mock.patch.object(driver_utils, 'get_services')
+    def test_on_finalize_no_network_in_kn_no_desc(self, m_get_svc,
+                                                  m_get_net_client):
+        crd_selector = mock.sentinel.crd_selector
+        self._delete_ns_sg_rules.return_value = [crd_selector]
+        m_get_svc.return_value = []
+        net_mock = mock.MagicMock()
+        net_id = '1612ffb1-ff7d-4590-bd9c-95aeb043705a'
+        net_mock.networks.return_value = [
+            os_network.Network(id=net_id, description=''),
+        ]
+        m_get_net_client.return_value = net_mock
+        self._handler.k8s.get.return_value = (
+            {'metadata': {'name': 'test-namespace', 'uid': net_id}})
+
+        kuryrnetwork.KuryrNetworkHandler.on_finalize(self._handler,
+                                                     self._kuryrnet_crd)
+
+        self._delete_network_pools.assert_not_called()
+        self._delete_ns_sg_rules.assert_called_once()
+        m_get_svc.assert_called_once()
+        self._handler._update_services.assert_called_once()
+        self._handler.k8s.remove_finalizer.assert_called_once()
+        self._handler.k8s.add_event.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    @mock.patch.object(driver_utils, 'get_services')
+    def test_on_finalize_no_network_in_kn_with_subnet(self, m_get_svc,
+                                                      m_get_net_client):
+        crd_selector = mock.sentinel.crd_selector
+        self._delete_ns_sg_rules.return_value = [crd_selector]
+        m_get_svc.return_value = []
+        net_id = 'db0b2c83-dae3-47ff-b4b0-9a19b7c15589'
+        ns_id = '0b6d6f0b-4e44-4a1b-a711-71ab6c79bee8'
+        subnet_id = 'a595fc4b-6885-48ff-b90c-d3f7aefd6d1a'
+        net_mock = mock.MagicMock()
+        net_mock.networks.return_value = [
+            os_network.Network(id=net_id, description=ns_id),
+        ]
+        net_mock.subnets.return_value = [os_subnet.Subnet(id=subnet_id)]
+        m_get_net_client.return_value = net_mock
+        self._handler.k8s.get.return_value = (
+            {'metadata': {'name': 'test-namespace', 'uid': ns_id}})
+
+        kuryrnetwork.KuryrNetworkHandler.on_finalize(self._handler,
+                                                     self._kuryrnet_crd)
+
+        self._delete_network_pools.assert_called_once()
+        self._delete_ns_sg_rules.assert_called_once()
+        m_get_svc.assert_called_once()
+        self._handler._update_services.assert_called_once()
+        self._handler.k8s.remove_finalizer.assert_called_once()
+        self._handler.k8s.add_event.assert_not_called()
+
+    @mock.patch('kuryr_kubernetes.clients.get_network_client')
+    @mock.patch.object(driver_utils, 'get_services')
+    def test_on_finalize_no_network_in_kn_with_no_ns_match(self, m_get_svc,
+                                                           m_get_net_client):
+        crd_selector = mock.sentinel.crd_selector
+        self._delete_ns_sg_rules.return_value = [crd_selector]
+        m_get_svc.return_value = []
+        net_id = 'db0b2c83-dae3-47ff-b4b0-9a19b7c15589'
+        ns_id = '0b6d6f0b-4e44-4a1b-a711-71ab6c79bee8'
+        subnet_id = 'a595fc4b-6885-48ff-b90c-d3f7aefd6d1a'
+        net_mock = mock.MagicMock()
+        net_mock.networks.return_value = [
+            os_network.Network(id=net_id, description=ns_id),
+        ]
+        net_mock.subnets.return_value = [os_subnet.Subnet(id=subnet_id)]
+        m_get_net_client.return_value = net_mock
+        self._handler.k8s.get.return_value = (
+            {'metadata': {'name': 'test-namespace', 'uid': net_id}})
+
+        kuryrnetwork.KuryrNetworkHandler.on_finalize(self._handler,
+                                                     self._kuryrnet_crd)
+
+        self._delete_network_pools.assert_not_called()
+        m_get_svc.assert_called_once()
+        self._handler._update_services.assert_called_once()
+        self._handler.k8s.remove_finalizer.assert_called_once()
+        self._handler.k8s.add_event.assert_not_called()
 
     @mock.patch.object(driver_utils, 'get_services')
     def test_on_finalize_no_sg_enforce(self, m_get_svc):
@@ -238,7 +336,6 @@ class TestKuryrNetworkHandler(test_base.TestCase):
 
         self._delete_ns_sg_rules.return_value = [crd_selector]
         m_get_svc.return_value = []
-        kubernetes = self.useFixture(k_fix.MockK8sClient()).client
         oslo_cfg.CONF.set_override('enforce_sg_rules',
                                    False,
                                    group='octavia_defaults')
@@ -253,7 +350,25 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._delete_ns_sg_rules.assert_called_once()
         m_get_svc.assert_not_called()
         self._handler._update_services.assert_not_called()
-        kubernetes.remove_finalizer.assert_called_once()
+        self._handler.k8s.remove_finalizer.assert_called_once()
+        self._handler.k8s.add_event.assert_not_called()
+
+    def test_on_finalize_finalizer_delete_ns_subnet_exception(self):
+        net_id = mock.sentinel.net_id
+        kns_crd = self._kuryrnet_crd.copy()
+        kns_crd['status'] = {'netId': net_id}
+        crd_selector = mock.sentinel.crd_selector
+
+        self._delete_ns_sg_rules.return_value = [crd_selector]
+        self._delete_namespace_subnet.side_effect = (k_exc.
+                                                     ResourceNotReady(kns_crd))
+
+        self.assertRaises(k_exc.ResourceNotReady,
+                          kuryrnetwork.KuryrNetworkHandler.on_finalize,
+                          self._handler, kns_crd)
+
+        self._delete_network_pools.assert_called_once_with(net_id)
+        self._delete_namespace_subnet.assert_called_once_with(kns_crd)
 
     @mock.patch.object(driver_utils, 'get_services')
     def test_on_finalize_finalizer_exception(self, m_get_svc):
@@ -264,8 +379,8 @@ class TestKuryrNetworkHandler(test_base.TestCase):
 
         self._delete_ns_sg_rules.return_value = [crd_selector]
         m_get_svc.return_value = []
-        kubernetes = self.useFixture(k_fix.MockK8sClient()).client
-        kubernetes.remove_finalizer.side_effect = k_exc.K8sClientException
+        k8s = self._handler.k8s
+        k8s.remove_finalizer.side_effect = k_exc.K8sClientException
 
         self.assertRaises(
             k_exc.K8sClientException,
@@ -277,4 +392,4 @@ class TestKuryrNetworkHandler(test_base.TestCase):
         self._delete_ns_sg_rules.assert_called_once()
         m_get_svc.assert_called_once()
         self._handler._update_services.assert_called_once()
-        kubernetes.remove_finalizer.assert_called_once()
+        k8s.remove_finalizer.assert_called_once()
