@@ -16,6 +16,7 @@ from unittest import mock
 
 from kuryr_kubernetes.controller.drivers import base as drivers
 from kuryr_kubernetes.controller.drivers import namespace_subnet as subnet_drv
+from kuryr_kubernetes.controller.drivers import node_subnets
 from kuryr_kubernetes.controller.drivers import utils as driver_utils
 from kuryr_kubernetes.controller.drivers import vif_pool
 from kuryr_kubernetes.controller.handlers import kuryrnetwork_population
@@ -46,12 +47,14 @@ class TestKuryrNetworkPopulationHandler(test_base.TestCase):
 
         self._handler = mock.MagicMock(
             spec=kuryrnetwork_population.KuryrNetworkPopulationHandler)
-        # NOTE(ltomasbo): The KuryrNet handler is associated to the usage of
-        # namespace subnet driver,
+        # NOTE(ltomasbo): The KuryrNetwork handler is associated to the usage
+        # of namespace subnet driver,
         self._handler._drv_subnets = mock.Mock(
             spec=subnet_drv.NamespacePodSubnetDriver)
         self._handler._drv_vif_pool = mock.MagicMock(
             spec=vif_pool.MultiVIFPool)
+        self._handler._drv_nodes_subnets = mock.MagicMock(
+            spec=node_subnets.ConfigNodesSubnets)
 
         self._get_namespace_subnet = (
             self._handler._drv_subnets.get_namespace_subnet)
@@ -78,12 +81,12 @@ class TestKuryrNetworkPopulationHandler(test_base.TestCase):
     @mock.patch.object(driver_utils, 'get_annotations')
     @mock.patch.object(driver_utils, 'get_namespace')
     @mock.patch.object(utils, 'get_nodes_ips')
-    def test_on_added(self, m_get_nodes_ips, m_get_ns, m_get_ann):
+    def test_on_present(self, m_get_nodes_ips, m_get_ns, m_get_ann):
         m_get_nodes_ips.return_value = ['node-ip']
         m_get_ns.return_value = mock.sentinel.ns
         m_get_ann.return_value = self._kuryrnet_crd['metadata']['name']
 
-        kuryrnetwork_population.KuryrNetworkPopulationHandler.on_added(
+        kuryrnetwork_population.KuryrNetworkPopulationHandler.on_present(
             self._handler, self._kuryrnet_crd)
 
         self._get_namespace_subnet.assert_called_once_with(
@@ -96,7 +99,7 @@ class TestKuryrNetworkPopulationHandler(test_base.TestCase):
 
     def test_on_added_no_subnet(self):
         kns = self._kuryrnet_crd.copy()
-        kns['status'] = {}
+        del kns['status']
         kuryrnetwork_population.KuryrNetworkPopulationHandler.on_added(
             self._handler, kns)
         self._get_namespace_subnet.assert_not_called()
