@@ -821,3 +821,26 @@ def cleanup_dead_networks():
             except os_exc.SDKException as ex:
                 LOG.warning('There was an issue with network "%s" '
                             'removal: %s', net, ex)
+
+
+def get_parent_port_id(vif_obj):
+    os_net = clients.get_network_client()
+    tags = []
+
+    if CONF.neutron_defaults.resource_tags:
+        tags = CONF.neutron_defaults.resource_tags
+
+    trunks = os_net.trunks(tags=tags)
+
+    for trunk in trunks:
+        for sp in trunk.sub_ports:
+            if sp['port_id'] == vif_obj.id:
+                return trunk.port_id
+
+    return None
+
+
+def get_parent_port_ip(port_id):
+    os_net = clients.get_network_client()
+    parent_port = os_net.get_port(port_id)
+    return parent_port.fixed_ips[0]['ip_address']
